import { InsertQueryNode } from './insert-query-node.js';
import { SelectQueryNode } from './select-query-node.js';
import { UpdateQueryNode } from './update-query-node.js';
import { DeleteQueryNode } from './delete-query-node.js';
import { WhereNode } from './where-node.js';
import { JoinNode } from './join-node.js';
import { SelectionNode } from './selection-node.js';
import { ReturningNode } from './returning-node.js';
import { OperationNode } from './operation-node.js';
import { ExplainNode } from './explain-node.js';
import { ExplainFormat } from '../util/explainable.js';
import { Expression } from '../expression/expression.js';
export declare type QueryNode = SelectQueryNode | InsertQueryNode | UpdateQueryNode | DeleteQueryNode;
declare type HasJoins = {
    joins?: ReadonlyArray<JoinNode>;
};
declare type HasWhere = {
    where?: WhereNode;
};
declare type HasReturning = {
    returning?: ReturningNode;
};
declare type HasExplain = {
    explain?: ExplainNode;
};
/**
 * @internal
 */
export declare const QueryNode: Readonly<{
    is(node: OperationNode): node is QueryNode;
    cloneWithWhere<T extends HasWhere>(node: T, operation: OperationNode): T;
    cloneWithOrWhere<T_1 extends HasWhere>(node: T_1, operation: OperationNode): T_1;
    cloneWithJoin<T_2 extends HasJoins>(node: T_2, join: JoinNode): T_2;
    cloneWithReturning<T_3 extends HasReturning>(node: T_3, selections: ReadonlyArray<SelectionNode>): T_3;
    cloneWithoutWhere<T_4 extends HasWhere>(node: T_4): T_4;
    cloneWithExplain<T_5 extends HasExplain>(node: T_5, format: ExplainFormat | undefined, options: Expression<any> | undefined): T_5;
}>;
export {};
