"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFunctionModule = void 0;
const expression_wrapper_js_1 = require("../expression/expression-wrapper.js");
const aggregate_function_node_js_1 = require("../operation-node/aggregate-function-node.js");
const function_node_js_1 = require("../operation-node/function-node.js");
const reference_parser_js_1 = require("../parser/reference-parser.js");
const select_parser_js_1 = require("../parser/select-parser.js");
const aggregate_function_builder_js_1 = require("./aggregate-function-builder.js");
function createFunctionModule() {
    const fn = (name, args) => {
        return new expression_wrapper_js_1.ExpressionWrapper(function_node_js_1.FunctionNode.create(name, (0, reference_parser_js_1.parseReferenceExpressionOrList)(args)));
    };
    return Object.assign(fn, {
        avg(column) {
            return new aggregate_function_builder_js_1.AggregateFunctionBuilder({
                aggregateFunctionNode: aggregate_function_node_js_1.AggregateFunctionNode.create('avg', (0, reference_parser_js_1.parseSimpleReferenceExpression)(column)),
            });
        },
        coalesce(value, ...otherValues) {
            return fn('coalesce', [value, ...otherValues]);
        },
        count(column) {
            return new aggregate_function_builder_js_1.AggregateFunctionBuilder({
                aggregateFunctionNode: aggregate_function_node_js_1.AggregateFunctionNode.create('count', (0, reference_parser_js_1.parseSimpleReferenceExpression)(column)),
            });
        },
        countAll(table) {
            return new aggregate_function_builder_js_1.AggregateFunctionBuilder({
                aggregateFunctionNode: aggregate_function_node_js_1.AggregateFunctionNode.create('count', (0, select_parser_js_1.parseSelectAll)(table)[0]),
            });
        },
        max(column) {
            return new aggregate_function_builder_js_1.AggregateFunctionBuilder({
                aggregateFunctionNode: aggregate_function_node_js_1.AggregateFunctionNode.create('max', (0, reference_parser_js_1.parseSimpleReferenceExpression)(column)),
            });
        },
        min(column) {
            return new aggregate_function_builder_js_1.AggregateFunctionBuilder({
                aggregateFunctionNode: aggregate_function_node_js_1.AggregateFunctionNode.create('min', (0, reference_parser_js_1.parseSimpleReferenceExpression)(column)),
            });
        },
        sum(column) {
            return new aggregate_function_builder_js_1.AggregateFunctionBuilder({
                aggregateFunctionNode: aggregate_function_node_js_1.AggregateFunctionNode.create('sum', (0, reference_parser_js_1.parseSimpleReferenceExpression)(column)),
            });
        },
    });
}
exports.createFunctionModule = createFunctionModule;
