"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlteredColumnBuilder = exports.AlterColumnBuilder = void 0;
const alter_column_node_js_1 = require("../operation-node/alter-column-node.js");
const data_type_node_js_1 = require("../operation-node/data-type-node.js");
const default_value_parser_js_1 = require("../parser/default-value-parser.js");
class AlterColumnBuilder {
    alterColumnNode;
    constructor(alterColumnNode) {
        this.alterColumnNode = alterColumnNode;
    }
    setDataType(dataType) {
        return new AlteredColumnBuilder(alter_column_node_js_1.AlterColumnNode.cloneWith(this.alterColumnNode, {
            dataType: data_type_node_js_1.DataTypeNode.create(dataType),
        }));
    }
    setDefault(value) {
        return new AlteredColumnBuilder(alter_column_node_js_1.AlterColumnNode.cloneWith(this.alterColumnNode, {
            setDefault: (0, default_value_parser_js_1.parseDefaultValueExpression)(value),
        }));
    }
    dropDefault() {
        return new AlteredColumnBuilder(alter_column_node_js_1.AlterColumnNode.cloneWith(this.alterColumnNode, {
            dropDefault: true,
        }));
    }
    setNotNull() {
        return new AlteredColumnBuilder(alter_column_node_js_1.AlterColumnNode.cloneWith(this.alterColumnNode, {
            setNotNull: true,
        }));
    }
    dropNotNull() {
        return new AlteredColumnBuilder(alter_column_node_js_1.AlterColumnNode.cloneWith(this.alterColumnNode, {
            dropNotNull: true,
        }));
    }
    /**
     * Simply calls the provided function passing `this` as the only argument. `$call` returns
     * what the provided function returns.
     */
    $call(func) {
        return func(this);
    }
}
exports.AlterColumnBuilder = AlterColumnBuilder;
/**
 * Allows us to force consumers to do something, anything, when altering a column.
 *
 * Basically, deny the following:
 *
 * ```ts
 * db.schema.alterTable('person').alterColumn('age', (ac) => ac)
 * ```
 *
 * Which would now throw a compilation error, instead of a runtime error.
 */
class AlteredColumnBuilder extends AlterColumnBuilder {
    toOperationNode() {
        return this.alterColumnNode;
    }
}
exports.AlteredColumnBuilder = AlteredColumnBuilder;
